@@ -1,4 +1,4 @@
-# Copyrights 2007-2011 by Mark Overmeer.
+# Copyrights 2007-2012 by Mark Overmeer.
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.00.
@@ -7,7 +7,7 @@ use strict;
 
 package XML::Compile::SOAP::Daemon::LWPutil;
 use vars '$VERSION';
-$VERSION = '3.00';
+$VERSION = '3.01';
 
 use base 'Exporter';
 
@@ -22,14 +22,14 @@ our @EXPORT = qw(
 );
 
 use Log::Report 'xml-compile-soap-daemon';
-use LWP;
-use HTTP::Status;
 use XML::Compile::SOAP::Util ':daemon';
+use LWP;
+use HTTP::Status qw/RC_OK RC_METHOD_NOT_ALLOWED RC_NOT_ACCEPTABLE/;
 
-sub lwp_add_header($$);
+sub lwp_add_header($$@);
 sub lwp_handle_connection($@);
-sub lwp_run_request($$;$);
-sub lwp_make_response($$$$);
+sub lwp_run_request($$;$$);
+sub lwp_make_response($$$$;$);
 sub lwp_action_from_header($);
 
 
@@ -45,9 +45,7 @@ BEGIN
     }
 }
 
-sub lwp_add_header($$)
-{   push @default_headers, @_;
-}
+sub lwp_add_header($$@) { push @default_headers, @_ }
 
 
 my $wsdl_response;
@@ -80,6 +78,7 @@ sub lwp_handle_connection($@)
     my $expires  = $args{expires};
     my $maxmsgs  = $args{maxmsgs};
     my $reqbonus = $args{reqbonus};
+    my $postproc = $args{postprocess};
 
     local $SIG{ALRM} = sub { die "timeout\n" };
 
@@ -90,7 +89,9 @@ sub lwp_handle_connection($@)
         alarm 0;
         $request or last;
 
-        my $response = lwp_run_request $request, $args{handler}, $connection;
+        my $response = lwp_run_request $request, $args{handler}
+          , $connection, $postproc;
+
         $connection->force_last_request if $maxmsgs==1;
         $connection->send_response($response);
 
@@ -100,8 +101,8 @@ sub lwp_handle_connection($@)
 }
 
 
-sub lwp_run_request($$;$)
-{   my ($request, $handler, $connection) = @_;
+sub lwp_run_request($$;$$)
+{   my ($request, $handler, $connection, $postproc) = @_;
 
 #   my $client   = $connection->peerhost;
     return $wsdl_response
@@ -128,14 +129,15 @@ sub lwp_run_request($$;$)
     my $charset  = $ct =~ m/\;\s*type\=(["']?)([\w-]*)\1/ ? $2: 'utf-8';
     my $xmlin    = $request->decoded_content(charset => $charset, ref => 1);
 
-    my ($status, $msg, $out) = $handler->($xmlin, $request, $action);
+    my ($status, $status_msg, $xml)
+      = $handler->($xmlin, $request, $action);
 
-    lwp_make_response $request, $status, $msg, $out;
+    lwp_make_response $request, $status, $status_msg, $xml, $postproc;
 }
 
 
-sub lwp_make_response($$$$)
-{   my ($request, $status, $msg, $body) = @_;
+sub lwp_make_response($$$$;$)
+{   my ($request, $status, $msg, $body, $postproc) = @_;
 
     my $response = HTTP::Response->new($status, $msg);
     $response->header(@default_headers);
@@ -151,6 +153,9 @@ sub lwp_make_response($$$$)
         $response->header(Content_Type => 'text/plain');
     }
 
+    $postproc->($request, $response, $status, \$s)
+        if $postproc;
+
     $response->content_ref(\$s);
     { use bytes; $response->header('Content-Length' => length $s); }
 
@@ -183,9 +188,12 @@ sub lwp_action_from_header($)
     {   return undef;
     }
 
-      !defined $action            ? undef
-    : $action =~ m/^\s*\"(.*?)\"/ ? $1
-    :                               $action;
+    defined $action or return;
+
+    $action =~ s/["'\s]//g;  # often wrong blanks and quotes
+    $action;
 }
 
+#------------------------------
+
 1;
